# Getting Started with Rails, Docker
## Setup
*  Clone this repository

```bash
# via SSH (Make sure that you have already finished the setup for the SSH keys)
git clone ssh://git@gitlab.discs.ateneo.edu:15316/mis-21-2018-intersession/1-getting-started-with-rails.git sample_app
# or
# via HTTPS
git clone https://gitlab.discs.ateneo.edu/mis-21-2018-intersession/1-getting-started-with-rails.git sample_app
```
**Note**: You can replace the name of the project e.g. ``sample_app`` to any preferred project name.

* Go inside the cloned repository.

```bash
cd sample_app
```

* Make sure that the ``start`` file is executable.

```bash
chmod +x start
```

* Make sure to re-initialize own Git repository.

```bash
rm -rf .git
git init
```

* Run this command to identify the current directory.

```bash
pwd
```
**Note**: Take note of the output of this command. This shall serve as an input value to the next commands.

* Build and start Docker containers.

```bash
docker-compose build
docker-compose up
```
**Note**: During the first run, this will take a while. Proceed with the next steps ONLY after a similar message shows up.

```bash
sample_app_web_1 exited with code 0
```

* Open another Terminal (Mac/Unix) / Powershell (Windows).
* Go inside the project directory.

```bash
cd <command output of the previous pwd command>
```

* Stop Docker containers.

```bash
docker-compose down
```

* Create a new Rails project.

```bash
docker-compose run web rails new . --force --database=postgresql
```
**Notes**: 
- You can remove ``--database=postgresql`` if a different database engine will be used.
- This will take a while. Proceed with the next step ONLY after a similar message shows up.

```bash
Bundle complete! 18 Gemfile dependencies, 78 gems now installed.
Use `bundle info [gemname]` to see where a bundled gem is installed.
         run  bundle exec spring binstub --all
* bin/rake: spring inserted
* bin/rails: spring inserted
```

* In one of the two opened Terminal (Mac/Unix) / Powershell (Windows), start again Docker containers.

```bash
docker-compose up
```

* Edit ``config/database.yml``.

```bash
default: &default
  adapter: postgresql
  encoding: unicode
  host: db
  username: <%= ENV['POSTGRES_USER'] %>
  password: <%= ENV['POSTGRES_PASSWORD'] %>
  pool: <%= ENV.fetch("RAILS_MAX_THREADS") { 5 } %>

development:
  <<: *default
  database: <%= "#{ENV['DATABASE_NAME']}_development" %>

test:
  <<: *default
  database: <%= "#{ENV['DATABASE_NAME']}_test" %>

production:
  <<: *default
  database: app_production
  username: app
  password: <%= ENV['RAILS_DATABASE_PASSWORD'] %>
```

* Create the ``development`` and ``test`` databases.

```bash
docker-compose run web rails db:create
```

## Adding [Better Errors](https://github.com/charliesome/better_errors) gem

* In ``Gemfile``, add these gems under ``development`` group.

```bash
group :development do
  ...
  
  gem 'better_errors'
  gem 'binding_of_caller'
end
```

* Run this command to setup Better Errors initializer file.

```bash
touch config/initializers/better_errors.rb
```

* Open ``config/initializers/better_errors.rb``.
* Add this code in ``config/initializers/better_errors.rb``

```bash
if Rails.env.development?
  BetterErrors::Middleware.allow_ip! "0.0.0.0/0"
end
```